/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIProperties.idl"

interface nsIInputStream;
interface nsIOutputStream;
interface nsISimpleEnumerator;

[scriptable, uuid(283EE646-1AEF-11D4-98B3-00C04fA0CE9A)]
interface nsIPropertyElement : nsISupports {
  attribute AUTF8String key;
  attribute AString value;
};

[scriptable, uuid(1A180F60-93B2-11d2-9B8B-00805F8A16D9)]
interface nsIPersistentProperties : nsIProperties
{
  /**
   * load a set of name/value pairs from the input stream
   * names and values should be in UTF8
   */
  void load(in nsIInputStream input);

  /**
   * output the values to the stream - results will be in UTF8
   */
  void save(in nsIOutputStream output, in AUTF8String header);

  /**
   * call subclass() to make future calls to load() set the properties
   * in this "superclass" instead
   */
  void subclass(in nsIPersistentProperties superclass);

  /**
   * get an enumeration of nsIPropertyElement objects,
   * which are read-only (i.e. setting properties on the element will
   * not make changes back into the source nsIPersistentProperties
   */
  nsISimpleEnumerator enumerate();

  /**
   * shortcut to nsIProperty's get() which retrieves a string value
   * directly (and thus faster)
   */
  AString getStringProperty(in AUTF8String key);

  /**
   * shortcut to nsIProperty's set() which sets a string value
   * directly (and thus faster). If the given property already exists,
   * then the old value will be returned
   */
  AString setStringProperty(in AUTF8String key, in AString value);
};


%{C++

#define NS_IPERSISTENTPROPERTIES_CID \
{ 0x2245e573, 0x9464, 0x11d2, \
  { 0x9b, 0x8b, 0x0, 0x80, 0x5f, 0x8a, 0x16, 0xd9 } }

#define NS_PERSISTENTPROPERTIES_CONTRACTID "@mozilla.org/persistent-properties;1"

%}

